@@ -65,9 +65,12 @@ class ChecklistMIP(object):
     def __init__(self, dataset, model_name = 'checklist', cost_func = '01',  one_feature_per_group = True,
                     lplus_weight = None, lminus_weight = None, compress = True):
         '''
-        dataset: object of type BinaryDataset  
-        cost_func: one of ['01', 'FNR', 'FPR'] 
-        compress: whether to compress samples that have the same features. Must be False for fairness constraints.
+            dataset: object of type BinaryDataset  
+            cost_func: one of ['01', 'FNR', 'FPR'] 
+            model_name: name of the checklist; only use for display purposes
+            one_feature_per_group: whether to enforce the constraint to select at most one variable from each feature group
+            lplus_weight, lminus_weight: custom weights for positive and negative errors, overrides cost_func weights
+            compress: whether to compress samples that have the same features. Must be False for fairness constraints.
         '''
         self.dataset = dataset
         target = dataset.target.copy()
@@ -79,7 +82,8 @@ class ChecklistMIP(object):
         self.model_name = model_name
         self.cost_func = cost_func
         self.constraints = [] # named constraints
-        self.solved = False     
+        self.solved = False 
+        self.built = False 
 
         self.prod = self._init(compress = compress) # processed dataset
 
@@ -109,6 +113,7 @@ class ChecklistMIP(object):
         df = pd.DataFrame(self.binarized_df)
         df['target'] = self.target
         if compress:
+            log.warn('Do not use compression when using fairness constraints!')
             log.info("Before compression: %s rows" % df.shape[0])        
             df = df.groupby(df.columns.tolist()).size().reset_index(name='weight_i')
             log.info("After compression: %s rows" % df.shape[0])
@@ -126,6 +131,14 @@ class ChecklistMIP(object):
         return ProcessedDataset(mat, target, I_plus, I_minus, n)
 
     def build_problem(self, N_constraint = None, M_constraint = None, use_indicator = False):  
+        '''
+            Builds the MIP by adding constraints and the objective.
+
+            Args:
+                N_constraint, M_constraint: Passing in these constraints here instead of using .add_constraint() will improve solution efficiency.
+                        Should be type MaxNumFeatureConstraint and MConstraint respectively.
+                use_indicator: whether to use indicator constraints instead of big-M constraints. Probably does not make a big difference.
+        '''
         (n_features, n_samples, n_samples_plus, n_samples_minus, n, I_plus, I_minus, mat) = (self.prod.n_features, 
             self.prod.n_samples, self.prod.n_samples_plus, self.prod.n_samples_minus, self.prod.n, self.prod.I_plus, self.prod.I_minus, self.prod.mat)
 
@@ -146,7 +159,7 @@ class ChecklistMIP(object):
         model = Cplex()
         model.objective.set_sense(model.objective.sense.minimize)
         constraints = model.linear_constraints
-        #self.add_constraint(LossConstraint('<=', min(len(I_plus), len(I_minus))))
+        
         names = {
             'lamb': print_vnames('lamb%d', n_features), # 1 if checklist includes item j
             'z': print_vnames('z%d', n_samples), # 1 if mistake on sample i is predicted correctly
@@ -252,17 +265,32 @@ class ChecklistMIP(object):
         self.model = model
         self.names = names
         self.N_min, self.N_max, self.M_min, self.M_max = N_min, N_max, M_min, M_max
+        self.built = True
 
     def add_constraint(self, constraint):
+        '''Adds a constraint to the MIP. Constraints can be found in constraints_cplex'''
         assert isinstance(constraint, Constraint)
+        assert not self.built, "Constraints need to be added before building the MIP."
         self.constraints.append(constraint)
 
     def solve(self, max_seconds = None, threads = 1, return_incumbents = False,
               display_progress = False, emphasis = 1, variableselect = 3,
               memory_limit = 1024*20, seed = 42):
 
-        if not hasattr(self, 'model'):
-            log.warn("Must build problem first!")
+        '''
+            Solves the MIP. 
+
+            Args: 
+                max_seconds: maximum number of seconds to solve for
+                threads: number of threads to use
+                return_incumbents: whether to store and return incumbent solutions
+                display_progress: print progress log to the screen
+                emphasis, variableselect: see https://www.ibm.com/docs/en/SSSA5P_12.8.0/ilog.odms.studio.help/pdf/usrcplex.pdf
+                memory_limit: memory limit in MB
+                seed: random seed
+        '''
+
+        assert self.built, "Must build MIP first!"
 
         if self.solved:
             log.warn('problem is already solved!')
@@ -307,7 +335,6 @@ class ChecklistMIP(object):
 
         return solution_info, progress_info, progress_incumbents
 
-
     def _add_stats_callback(self, sol_idx, store_solutions = False):
         if not hasattr(self, '_stats_callback'):
             min_idx = min(sol_idx)
@@ -363,12 +390,14 @@ class ChecklistMIP(object):
         return info
 
     def add_initial_solution_from_mip(self, other_mip):
+        '''Adds an initial solution to the MIP in the form of a solved MIP.'''
         assert(isinstance(other_mip, ChecklistMIP))
         var_names = other_mip.model.variables.get_names()
         var_values = other_mip.model.solution.get_values(var_names)
         self.model.MIP_starts.add([var_names, var_values], self.model.MIP_starts.effort_level.check_feasibility)    
         
     def add_initial_solution_from_checklist(self, checklist):
+        '''Adds an initial solution to the MIP in the form of a checklist.'''
         assert(isinstance(checklist, Checklist))
         for i in checklist.column_names:
             if i not in self.remaining_features:
@@ -377,6 +406,7 @@ class ChecklistMIP(object):
         self.add_initial_solution_from_weights(checklist.lamb, checklist.M) 
         
     def add_initial_solution_from_weights(self, lamb, M):
+        '''Adds an initial solution to the MIP by providing the lambda vector and M directly.'''
         assert(len(lamb) == len(self.names['lamb']))
         assert(M <= self.M_max and M >= self.M_min), (M, self.M_max, self.M_min)
         assert(i in [0, 1, 0., 1.] for i in lamb)
